#!/bin/bash
echo =============================================================================
echo "Test of your MACSIMUS installation."
echo "This is your current environment:"
echo "  BLENDPATH=$BLENDPATH"
echo "  GUI=$GUI"
echo "  PATH=$PATH"
echo "If undefined, try install.sh again and follow the instructions!"
echo
echo "-------------------------------------------------------------"
echo "Now we will run blend to create the force-field file SPCE.ble"
echo "Type ENTER -- a window with a water molecule should appear."
echo "To continue:"
echo "  type '.' in the window with the molecule"
echo "  or [finish] in the control panel"
echo "(if you fail, you will get error \"table species not found\" later)"
read

[ -e spce.ble ] && rm spce.ble
[ -e SPCE.mol ] && rm SPCE.mol SPCE.gol SPCE.plb

if ! type blend ; then
  echo bad MACSIMUS installation - blend not found!
  echo check whether blend has been compiled and is in your path
  echo your current PATH=$PATH
  exit
fi
blend -g -o spce -h SPCE.che
echo
if ! fgrep "*1-4" spce.ble ; then
  echo "You have not closed blend correctly - incorrect spce.ble was generated!"
  echo "But do not worry, I'll fix it in 2 s.."
  sleep 2
  rm SPCE.{mol,gol}
  blend -o spce -h SPCE.che
fi

[ -e spce.def ] && rm spce.def SPCE200.*
cat > spce.def <<EOF
N[0]=216
cutoff=9.3 LJcutoff=cutoff
rho=1000
equalize.mol=0.8 noint=5 h=0.01/noint
thermostat="Berendsen" tau.T=0.2
dt.plb=.1
dt.prt=.1
;
EOF
cat > SPCE200.get <<EOF
init="crystal"
no=200;
EOF
echo "--- Input file spce.def: ---"
cat spce.def
echo "--- Input file SPCE200.get: ---"
cat SPCE200.get
echo
echo "Type ENTER to start simulation (will finish in ~10 s on a modern computer)..."
read
if ! type cook ; then
  echo bad MACSIMUS installation - cook not found!
  echo check whether cook has been compiled and is in your path
  echo your current PATH=$PATH
  exit
fi
if ! cook spce SPCE200 ; then
  echo "cook failed or not stopped correctly"
  echo "try $PWD/$0 again"
  exit
fi
echo
echo "Type ENTER to show the trajectory..."
read
if ! type show ; then
  echo bad MACSIMUS installation - show not found!
  echo check whether show has been compiled and is in your path
  echo your current PATH=$PATH
  exit
fi
show SPCE200 -I%i
echo
echo "Type ENTER to show selected quantities as the function of time:"
echo "  - three graphs should appear; type 'K' in any one to kill them all"
read
if ! type showcp ; then
  echo bad MACSIMUS installation - showcp not found!
  echo check whether showcp has been compiled and is in your path
  echo your current PATH=$PATH
  exit
fi
if ! type plot ; then
  echo bad MACSIMUS installation - plot not found!
  echo check whether plot has been compiled and is in your path
  echo your current PATH=$PATH
  exit
fi
showcp -p SPCE200 Etot Tkin P
echo "That's it!"
echo "Erase all files created during running this test (y/N)?"
read
if [ "$REPLY" == "y" ] ; then
  rm SPCE200.* spce.* SPCE.plb SPCE.mol SPCE.gol
fi
